@@ -1,9 +1,17 @@
+//! Authentication schemes for use with `hyper`.
+//! Use the `hyper_async` feature to enable these.
+
 use std::borrow::Cow;
 use std::sync::Arc;
 
-use crate::credential::{AuthenticationCredentialToken, AuthenticationCredentialUsernamePassword};
-use crate::AuthenticationScheme;
+use crate::credential::{
+    AuthenticationCredentialToken, AuthenticationCredentialUsernamePassword, HttpRealmCredential,
+};
+use crate::{AuthenticError, AuthenticationScheme};
 
+/// No authentication scheme
+///
+/// Identical to not using `authentic` but allows minimal code changes when changing schemes.
 pub struct NoAuthentication;
 
 impl NoAuthentication {
@@ -20,6 +28,7 @@ impl AuthenticationScheme for NoAuthentication {
     type Error = hyper::Error;
 }
 
+/// Authentication using a token in a specified haeader.
 pub struct HeaderAuthentication<Credential> {
     header_name: Cow<'static, [u8]>,
     credential: Arc<Credential>,
@@ -51,6 +60,7 @@ where
     }
 }
 
+/// Authentication using HTTP Basic authentication on the initial call without waiting for a challenge.
 pub struct BasicAuthentication<Credential> {
     credential: Arc<Credential>,
 }
@@ -84,15 +94,17 @@ where
     }
 }
 
+/// Authentication using HTTP Basic authentication to respond to a challenge.
+///
+/// This currently only supports Basic authentication.
+///
+/// This limitation is expected to be removed in a future version.
 pub struct HttpAuthentication<Credential> {
-    credential: Arc<Credential>,
+    credential: Arc<HttpRealmCredential<Credential>>,
 }
 
-impl<Credential> HttpAuthentication<Credential>
-where
-    Credential: AuthenticationCredentialUsernamePassword + 'static,
-{
-    pub fn new(credential: &Arc<Credential>) -> Self {
+impl<Credential> HttpAuthentication<Credential> {
+    pub fn new(credential: &Arc<HttpRealmCredential<Credential>>) -> Self {
         Self {
             credential: credential.clone(),
         }
@@ -111,21 +123,41 @@ where
     fn switch(
         &mut self,
         response: &Self::Response,
-    ) -> Option<
-        Box<
-            dyn AuthenticationScheme<
-                Builder = Self::Builder,
-                Request = Self::Request,
-                Response = Self::Response,
-                Error = Self::Error,
+    ) -> Result<
+        Option<
+            Box<
+                dyn AuthenticationScheme<
+                    Builder = Self::Builder,
+                    Request = Self::Request,
+                    Response = Self::Response,
+                    Error = Self::Error,
+                >,
             >,
         >,
+        AuthenticError,
     > {
         if response.status() == ::http::StatusCode::UNAUTHORIZED {
-            // TODO: handle www-authenticate, including realms and Digest
-            Some(Box::new(BasicAuthentication::new(&self.credential)))
+            let pw_client = ::http_auth::PasswordClient::try_from(
+                response
+                    .headers()
+                    .get_all(::hyper::header::WWW_AUTHENTICATE),
+            )
+            .map_err(AuthenticError::Other)?;
+            match pw_client {
+                http_auth::PasswordClient::Basic(client) => {
+                    let realm = client.realm();
+                    match self.credential.get_credential(realm) {
+                        Some(credential) => {
+                            Ok(Some(Box::new(BasicAuthentication::new(credential))))
+                        }
+                        None => Err(AuthenticError::UnknownRealm(realm.to_owned())),
+                    }
+                }
+                http_auth::PasswordClient::Digest(_) => todo!(),
+                _ => todo!(),
+            }
         } else {
-            None
+            Ok(None)
         }
     }
 }
