@@ -1,8 +1,10 @@
 #![cfg(feature = "hyper")]
 
-use authentic::credential::UsernamePasswordCredential;
-use authentic::hyper::{HttpAuthentication, BasicAuthentication};
-use authentic::{AuthenticateBuilder, AuthenticationScheme, AuthenticationStep, Scheme};
+use std::collections::HashMap;
+
+use authentic::credential::{HttpRealmCredential, UsernamePasswordCredential};
+use authentic::hyper::{BasicAuthentication, HttpAuthentication};
+use authentic::{AuthenticationScheme, AuthenticationStep, WithAuthentication};
 use http::StatusCode;
 use hyper::Client;
 use hyper_tls::HttpsConnector;
@@ -19,7 +21,7 @@ async fn test_basic_authentication(
 
     let mut status_codes = Vec::new();
 
-    let response = loop {
+    let _response = loop {
         while let Some(auth_step) = scheme.step() {
             match auth_step {
                 AuthenticationStep::Request(request) => {
@@ -43,10 +45,9 @@ async fn test_basic_authentication(
 
         status_codes.push(response.status());
 
-        scheme = match scheme.switch(&response) {
-            Some(boxed) => Scheme::Boxed(boxed),
-            None => break response,
-        };
+        if scheme.has_completed(&response)? {
+            break response;
+        }
     };
 
     assert_eq!(status_codes, [StatusCode::OK]);
@@ -56,17 +57,21 @@ async fn test_basic_authentication(
 
 /// Test basic authentication, passing the username and password in response to a 401 challenge.
 #[::tokio::test]
-async fn test_basic_challenge(
-) -> Result<(), Box<dyn std::error::Error + Send + Sync + 'static>> {
+async fn test_basic_challenge() -> Result<(), Box<dyn std::error::Error + Send + Sync + 'static>> {
     let https = HttpsConnector::new();
     let client = Client::builder().build::<_, ::hyper::Body>(https);
 
-    let credential = UsernamePasswordCredential::new("username".into(), "password".into());
+    let mut realm_credentials = HashMap::new();
+    realm_credentials.insert(
+        "Fake Realm".into(),
+        UsernamePasswordCredential::new("username".into(), "password".into()),
+    );
+    let credential = HttpRealmCredential::new(realm_credentials);
     let mut scheme = HttpAuthentication::new(&credential).into_scheme();
 
     let mut status_codes = Vec::new();
 
-    let response = loop {
+    let _response = loop {
         while let Some(auth_step) = scheme.step() {
             match auth_step {
                 AuthenticationStep::Request(request) => {
@@ -90,10 +95,9 @@ async fn test_basic_challenge(
 
         status_codes.push(response.status());
 
-        scheme = match scheme.switch(&response) {
-            Some(boxed) => Scheme::Boxed(boxed),
-            None => break response,
-        };
+        if scheme.has_completed(&response)? {
+            break response;
+        }
     };
 
     assert_eq!(status_codes, [StatusCode::UNAUTHORIZED, StatusCode::OK]);
